@@ -127,7 +127,8 @@ static guint fr_archive_signals[LAST_SIGNAL] = { 0 };
 
 static void
 _fr_archive_set_file (FrArchive *self,
-		      GFile     *file)
+		      GFile     *file,
+		      gboolean   reset_mime_type)
 {
 	if (self->priv->file != NULL) {
 		g_object_unref (self->priv->file);
@@ -136,7 +137,8 @@ _fr_archive_set_file (FrArchive *self,
 	if (file != NULL)
 		self->priv->file = g_object_ref (file);
 
-	self->mime_type = NULL;
+	if (reset_mime_type)
+		self->mime_type = NULL;
 
 	g_object_notify (G_OBJECT (self), "file");
 }
@@ -149,7 +151,7 @@ _fr_archive_set_uri (FrArchive  *self,
 	GFile *file;
 
 	file = (uri != NULL) ? g_file_new_for_uri (uri) : NULL;
-	_fr_archive_set_file (self, file);
+	_fr_archive_set_file (self, file, TRUE);
 
 	_g_object_unref (file);
 }
@@ -167,7 +169,7 @@ fr_archive_set_property (GObject      *object,
 
         switch (property_id) {
         case PROP_FILE:
-        	_fr_archive_set_file (self, g_value_get_object (value));
+        	_fr_archive_set_file (self, g_value_get_object (value), TRUE);
         	break;
 	case PROP_MIME_TYPE:
 		fr_archive_set_mime_type (self, g_value_get_string (value));
@@ -547,24 +549,6 @@ fr_archive_set_stoppable (FrArchive *archive,
 /* -- fr_archive_new_for_creating -- */
 
 
-static const char *
-get_mime_type_from_filename (GFile *file)
-{
-	const char *mime_type = NULL;
-	char       *uri;
-
-	if (file == NULL)
-		return NULL;
-
-	uri = g_file_get_uri (file);
-	mime_type = get_mime_type_from_extension (_g_filename_get_extension (uri));
-
-	g_free (uri);
-
-	return mime_type;
-}
-
-
 static FrArchive *
 create_archive_for_mime_type (GType          archive_type,
 			      GFile         *file,
@@ -594,13 +578,27 @@ FrArchive *
 fr_archive_create (GFile      *file,
 		   const char *mime_type)
 {
-	GType      archive_type;
-	FrArchive *archive;
-	GFile     *parent;
+	FrArchiveCaps  requested_capabilities;
+	GType          archive_type;
+	FrArchive     *archive;
+	GFile         *parent;
 
 	if (mime_type == NULL)
-		mime_type = get_mime_type_from_filename (file);
-	archive_type = get_archive_type_from_mime_type (mime_type, FR_ARCHIVE_CAN_WRITE);
+		mime_type = _g_mime_type_get_from_filename (file);
+
+	/* try with the CAN_READ capability as well, this way we give
+	 * priority to the commands that can read and write over commands
+	 * that can only create a specific file format. */
+
+	requested_capabilities = FR_ARCHIVE_CAN_READ_WRITE;
+	archive_type = get_archive_type_from_mime_type (mime_type, requested_capabilities);
+
+	/* if no command was found, remove the read capability and try again */
+
+	if (archive_type == 0) {
+		requested_capabilities ^= FR_ARCHIVE_CAN_READ;
+		archive_type = get_archive_type_from_mime_type (mime_type, requested_capabilities);
+	}
 
 	archive = create_archive_for_mime_type (archive_type,
 						file,
@@ -659,90 +657,21 @@ open_data_complete_with_error (OpenData *load_data,
 }
 
 
-static const char *
-get_mime_type_from_magic_numbers (char  *buffer,
-				  gsize  buffer_size)
-{
-#if ENABLE_MAGIC
-
-	static magic_t magic = NULL;
-
-	if (magic == NULL) {
-		magic = magic_open (MAGIC_MIME_TYPE);
-		if (magic != NULL)
-			magic_load (magic, NULL);
-		else
-			g_warning ("unable to open magic database");
-	}
-
-	if (magic != NULL) {
-		const char * mime_type;
-
-		mime_type = magic_buffer (magic, buffer, buffer_size);
-		if (mime_type)
-			return mime_type;
-
-		g_warning ("unable to detect filetype from magic: %s", magic_error (magic));
-	}
-
-#else
-
-	static const struct magic {
-		const unsigned int off;
-		const unsigned int len;
-		const char * const id;
-		const char * const mime_type;
-	}
-	magic_ids [] = {
-		/* magic ids taken from magic/Magdir/archive from the file-4.21 tarball */
-		{ 0,  6, "7z\274\257\047\034",                   "application/x-7z-compressed" },
-		{ 7,  7, "**ACE**",                              "application/x-ace"           },
-		{ 0,  2, "\x60\xea",                             "application/x-arj"           },
-		{ 0,  3, "BZh",                                  "application/x-bzip2"         },
-		{ 0,  2, "\037\213",                             "application/x-gzip"          },
-		{ 0,  4, "LZIP",                                 "application/x-lzip"          },
-		{ 0,  9, "\x89\x4c\x5a\x4f\x00\x0d\x0a\x1a\x0a", "application/x-lzop",         },
-		{ 0,  4, "Rar!",                                 "application/x-rar"           },
-		{ 0,  4, "RZIP",                                 "application/x-rzip"          },
-		{ 0,  6, "\3757zXZ\000",                         "application/x-xz"            },
-		{ 20, 4, "\xdc\xa7\xc4\xfd",                     "application/x-zoo",          },
-		{ 0,  4, "PK\003\004",                           "application/zip"             },
-		{ 0,  8, "PK00PK\003\004",                       "application/zip"             },
-		{ 0,  4, "LRZI",                                 "application/x-lrzip"         },
-	};
-
-	int  i;
-
-	for (i = 0; i < G_N_ELEMENTS (magic_ids); i++) {
-		const struct magic * const magic = &magic_ids[i];
-
-		if ((magic->off + magic->len) > buffer_size)
-			g_warning ("buffer underrun for mime-type '%s' magic", magic->mime_type);
-		else if (! memcmp (buffer + magic->off, magic->id, magic->len))
-			return magic->mime_type;
-	}
-
-#endif
-
-	return NULL;
-}
-
-
 static FrArchive *
 create_archive_to_load_archive (GFile      *file,
 			        const char *mime_type)
 {
-	FrArchiveCaps requested_capabilities = FR_ARCHIVE_CAN_DO_NOTHING;
+	FrArchiveCaps requested_capabilities;
 	GType         archive_type;
 
 	if (mime_type == NULL)
 		return FALSE;
 
-	/* try with the WRITE capability even when loading, this way we give
+	/* try with the CAN_WRITE capability even when loading, this way we give
 	 * priority to the commands that can read and write over commands
 	 * that can only read a specific file format. */
 
-	requested_capabilities |= FR_ARCHIVE_CAN_READ_WRITE;
+	requested_capabilities = FR_ARCHIVE_CAN_READ_WRITE;
 	archive_type = get_archive_type_from_mime_type (mime_type, requested_capabilities);
 
 	/* if no command was found, remove the write capability and try again */
@@ -777,29 +706,23 @@ open_archive_ready_cb (GObject      *source_object,
 
 
 static void
-open_archive_stream_ready_cb (GObject      *source_object,
+open_archive_buffer_ready_cb (GObject      *source_object,
 			      GAsyncResult *result,
 			      gpointer      user_data)
 {
 	OpenData         *open_data = user_data;
-	GFileInputStream *istream = G_FILE_INPUT_STREAM (source_object);
 	GError           *error = NULL;
-	gssize            bytes_read;
 	const char       *mime_type;
 	gboolean          result_uncertain;
 	FrArchive        *archive;
-	char             *local_mime_type, *uri;
+	char             *uri;
+	char             *local_mime_type;
 
-	bytes_read = g_input_stream_read_finish (G_INPUT_STREAM (istream), result, &error);
-	if (bytes_read == -1) {
-		g_object_unref (istream);
+	if (! _g_file_load_buffer_finish (open_data->file, result, &open_data->buffer, &open_data->buffer_size, &error)) {
 		open_data_complete_with_error (open_data, error);
 		return;
 	}
 
-	open_data->buffer_size = bytes_read;
-	g_object_unref (istream);
-
 	archive = NULL;
 	uri = g_file_get_uri (open_data->file);
 	local_mime_type = g_content_type_guess (uri, (guchar *) open_data->buffer, open_data->buffer_size, &result_uncertain);
@@ -808,10 +731,10 @@ open_archive_stream_ready_cb (GObject      *source_object,
 		archive = create_archive_to_load_archive (open_data->file, mime_type);
 	}
 	if (archive == NULL) {
-		mime_type = get_mime_type_from_magic_numbers (open_data->buffer, open_data->buffer_size);
+		mime_type = _g_mime_type_get_from_content (open_data->buffer, open_data->buffer_size);
 		archive = create_archive_to_load_archive (open_data->file, mime_type);
 		if (archive == NULL) {
-			mime_type = get_mime_type_from_filename (open_data->file);
+			mime_type = _g_mime_type_get_from_filename (open_data->file);
 			archive = create_archive_to_load_archive (open_data->file, mime_type);
 			if (archive == NULL) {
 				error = g_error_new_literal (FR_ERROR,
@@ -840,31 +763,6 @@ open_archive_stream_ready_cb (GObject      *source_object,
 }
 
 
-static void
-open_archive_read_ready_cb (GObject      *source_object,
-			    GAsyncResult *result,
-			    gpointer      user_data)
-{
-	OpenData         *open_data = user_data;
-	GFileInputStream *istream;
-	GError           *error = NULL;
-
-	istream = g_file_read_finish (G_FILE (source_object), result, &error);
-	if (istream == NULL) {
-		open_data_complete_with_error (open_data, error);
-		return;
-	}
-
-	g_input_stream_read_async (G_INPUT_STREAM (istream),
-				   open_data->buffer,
-				   open_data->buffer_size,
-				   G_PRIORITY_DEFAULT,
-				   open_data->cancellable,
-				   open_archive_stream_ready_cb,
-				   open_data);
-}
-
-
 void
 fr_archive_open (GFile               *file,
 		 GCancellable        *cancellable,
@@ -882,19 +780,19 @@ fr_archive_open (GFile               *file,
 						       callback,
 						       user_data,
 						       fr_archive_open);
-	open_data->buffer_size = BUFFER_SIZE_FOR_PRELOAD;
-	open_data->buffer = g_new (char, open_data->buffer_size);
+	open_data->buffer_size = 0;
+	open_data->buffer = NULL;
         g_simple_async_result_set_op_res_gpointer (open_data->result,
         					   open_data,
                                                    (GDestroyNotify) open_data_free);
 
         /* load a few bytes to guess the archive type */
 
-	g_file_read_async (open_data->file,
-			   G_PRIORITY_DEFAULT,
-			   open_data->cancellable,
-			   open_archive_read_ready_cb,
-			   open_data);
+	_g_file_load_buffer_async (open_data->file,
+				   BUFFER_SIZE_FOR_PRELOAD,
+				   open_data->cancellable,
+				   open_archive_buffer_ready_cb,
+				   open_data);
 }
 
 
@@ -2014,7 +1912,7 @@ fr_archive_set_multi_volume (FrArchive *self,
 			     GFile     *file)
 {
 	self->multi_volume = TRUE;
-	_fr_archive_set_file (self, file);
+	_fr_archive_set_file (self, file, FALSE);
 }
 
 
@@ -2030,7 +1928,7 @@ fr_archive_change_name (FrArchive  *archive,
 
 	parent = g_file_get_parent (archive->priv->file);
 	file = g_file_get_child (parent, name);;
-	_fr_archive_set_file (archive, file);
+	_fr_archive_set_file (archive, file, FALSE);
 
 	g_object_unref (parent);
 	g_object_unref (file);
@@ -2250,9 +2148,9 @@ _g_file_is_archive (GFile *file)
 		uri = g_file_get_uri (file);
 		mime_type = g_content_type_guess (uri, (guchar *) buffer, buffer_size, &result_uncertain);
 		if (result_uncertain) {
-			mime_type = get_mime_type_from_magic_numbers (buffer, buffer_size);
+			mime_type = _g_mime_type_get_from_content (buffer, buffer_size);
 			if (mime_type == NULL)
-				mime_type = get_mime_type_from_filename (file);
+				mime_type = _g_mime_type_get_from_filename (file);
 		}
 
 		if (mime_type != NULL) {
